#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <asm/ioctls.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <termios.h>
#include <errno.h>
#include <sys/mman.h>



#define MAXLINE 80

//microsecond delays
#define enterDelay 100000
#define writeDelay 100000

//#define DEBUG   0

int numSquirts;
int startIP[4];
int testData[8];
int blinkRate; 

typedef struct {
  void *next;
  int mods;
  char value[1];
} key_decode_type;


int keyfd,ttyfd;



int init_serial()
/* This starts up the Twiddler driver, reading in configure files, opening
   the serial port, and all that good stuff. */
{
  struct termios term;

  //if ((keyfd=open("/dev/ttyS0",O_RDWR | O_NOCTTY | O_NONBLOCK))<0)
  if ((keyfd=open("/dev/ttyS0",O_RDWR))<0)
    return -1;
  if ((tcgetattr(keyfd,&term))<0) {
    fprintf(stderr,"%d - ",errno);
    fprintf(stderr,"Couldn't get tty info!\n");
    return (-1);
  }
  term.c_cflag &= ~CBAUD; /* init baud rate */
  term.c_cflag |= B9600;
  tcsetattr(keyfd,TCSAFLUSH,&term);
  tcflush(keyfd,TCIOFLUSH);
  tcgetattr(keyfd,&term);

  /* put in raw data mode */
  term.c_lflag &= ~(ICANON | ECHO | IEXTEN | ISIG);
  term.c_iflag &= ~(BRKINT | ICRNL | INPCK | ISTRIP);
  term.c_cflag &= ~(CSIZE | PARENB);
  term.c_oflag &= ~OPOST;
  term.c_oflag &= ~(OPOST | OCRNL);
  term.c_oflag &= ~(OPOST | ONLCR);
  term.c_cflag |= CS8; /* set to 8 stop bits */
  term.c_cflag |= CLOCAL;
  term.c_cc[VMIN]=1;
  term.c_cc[VTIME]=0;
  cfsetospeed(&term, B9600);
  cfsetispeed(&term, B9600);

  tcsetattr(keyfd,TCSAFLUSH,&term);
  tcflush(keyfd,TCIOFLUSH);
//  tcflow(keyfd,TCOOFF);
  return 0;
}


void readline(char *t) {
  int i = 0;
  int count;
  unsigned char chr;
  int done = 0; 

  while (!done){

    count=read(keyfd,&chr,1);

    if (count != -1) {
//      if ((chr == '\n') || (chr == '\r')) {
      if ((chr == '\n') ) {
	done = 1;
	*(t+i) = 0;
      } else {
#ifdef DEBUG 
	printf("\t-> %c (%d)\n",chr,chr);
#endif
	*(t+i) = chr;
	i++;
      }
    }
  }
}


int writeCurrentSquirt(int squirt) {

  int i = 0, j=0;
  int valid; 
  char s[MAXLINE];  
  char h[MAXLINE];  
  unsigned char chr;
  int count;
  int writeData[8];


  writeData[0] = startIP[0];
  writeData[1] = startIP[1];
  writeData[2] = startIP[2];
  writeData[3] = startIP[3];
  writeData[4] = startIP[3];
  writeData[5] = startIP[3];
  writeData[6] = startIP[3];
  writeData[7] = blinkRate;

  /*   wait for squirt startup on serial (Sv2) */

  printf("Waiting for Squirt %d\n",squirt);
  readline(s);
  printf(" -%s\n",s);
    
  while (strncmp(s,"Sv2",3)) {
    readline(s);
    printf(" -%s\n",s);	
  }
    
  /* read current value (first program (ffs) vs reprog) */
  printf("Reading current settings\n");
  readline(s);
  printf(" -%s\n",s);

  //Delay before writing.
  printf("Delay\n");
  usleep(enterDelay);

  // Send return 
  printf("Sending interrupt\n");
  s[0] = '.';
  s[1] = 13;
  s[2] = 10;
  count = write(keyfd,s,1);  

#ifdef DEBUG
  printf(" I wrote %d\n",count);
#endif

  //Get ACK. 
  printf("Reading ACK\n");
  readline(s);
  printf(" -%s\n",s);

  // send program code (in decimal)
  //This is really ugly, but the PIC is
  //blown away if we transmit at full speed!

  printf("Writing data\n");

  //For each byte of data
  for (i=0;i<8;i++) {

    sprintf(s,"%03d",writeData[i]);

    //write each digit with delay
    for(j=0;j<3;j++) {
      usleep(writeDelay);
      h[0] = s[j];
      h[1] = 0;
      count = write(keyfd,h,1);  
    }

    //write proper termination
    s[0] = 10;
    s[1] = 13;
    s[2] = 0;
    count = write(keyfd,s,2);  

#ifdef DEBUG    
    printf("%s",s);
    printf("\n--\n");
    printf("  (I wrote %d)\n",count);
#endif
  } //end writing data

  // read back character echos
  readline(s);
#ifdef DEBUG
  printf(" -%s\n",s);
#endif DEBUG

  //read back result code
  printf("Verifying settings\n");
  readline(s);
  printf(" -%s\n",s);

  //test result code. 
  sscanf(s,"%X,%X,%X,%X,%X,%X,%X,%X.",&testData[0],&testData[1],&testData[2],&testData[3],&testData[4],&testData[5],&testData[6],&testData[7]);

  valid = 1;

  for (i=0;i<8;i++){
    if (testData[i] != writeData[i])
      valid = 0;
  }
  
  if (valid) {
    printf("SUCCESS\n");
  } else {
    printf("\bFAIL FAIL FAIL FAIL\n");
  }

  return(valid);
}


int main()
{
  int currentSquirt;


  init_serial();


  printf("How many Squirt2s do you want to program?\n");

  scanf("%d",&numSquirts);

  printf(" Starting IP address\n");



  /*Rochester tags 128.151.23.x */

  /* 
  startIP[0] = 10;
  startIP[1] = 20;
  startIP[2] = 30;
  startIP[3] = 40;  */

  scanf("%d.%d.%d.%d",&startIP[0],&startIP[1],&startIP[2],&startIP[3]);

  printf("using %d.%d.%d.%d\n",startIP[0],startIP[1],startIP[2],startIP[3]);

  printf(" Blink Rate?\n");

  scanf("%d",&blinkRate);

  printf("Programming %d Squirts, starting at %d.%d.%d.%d\n",numSquirts,startIP[0],startIP[1],startIP[2],startIP[3]);

  for (currentSquirt=1;currentSquirt<=numSquirts;currentSquirt++) {
    writeCurrentSquirt(currentSquirt);
    startIP[3]++;
    printf("\n\n");
  }

  close(keyfd);
  return 0;
}






