// File: squirt2b.c
//
// Testing the Rev 2 version of the Squirt board.
//  (This is the most recent version, changing squirt2a.c to be slightly more efficient)
//    
// 
//
//Ocillator Trim
// Pic-1: EC 
// Pic-2: F8
// Pic-3: D4
// Pic-4: F0
// Pic-5: B0
//// Found sitting out, no timing info!
// Pic-6: FF --???
// Pic-7: FF --???
/// Red
/// R-1: D4


#include <12ce519.h>
#include <defs_12c.h>

//**********************************
//* DEVICE CUSTOMIZATION DONE HERE *
//**********************************
//

//Print Debugging information to RS232?
//#define DEBUG

//If the device is OTP, we do not need to load the OSCCAL settings
// (In fact, loading the (incorrect) settings is VERY BAD!)
#define OTP

//OSCCAL SETTING! MAKE SURE TO READ IT AND CHANGE THIS TO MATCH
//BEFORE PROGRAMMING ANY CHIPS! (ignore for OTP chips)
#define OSC_VALUE 0xD4

//ID to be programmed into Chip's Hardware ID
#define DEVICE_ID 000

//****************************
//* END DEVICE CUSTOMIZATION *
//****************************

// Configure PIC to use: HS clock, no Watchdog Timer, 
// no code protection, and INTERNAL (ie, no) master clear
//
#fuses INTRC,WDT,NOPROTECT,NOMCLR

// This statement puts the device ID into the chip's internal device id word.
#id DEVICE_ID
 
// These Statements are required to properly program the 12CE519's clock offset.
//  Change the device-specific offset at the top (OSC_VALUE)
#ifndef OTP
 #byte OSC_LOC = 5
 #define SET_OSCILLATOR  OSC_LOC=OSC_VALUE
#endif

// Tell compiler clock is 4MHz.  This is required for the
// software delay loops that compose delay_ms() and for 
// all serial I/O (such as printf() and friends)
#use DELAY(clock=4000000)

#use fast_io(B)

//Pin_B3 is input only...
#define IRX_B_TRIS      0b00001000

#define LED        PIN_B1
//Yes, this is ALSO RS232 XMIT!  

#define IR1        PIN_B0
#define IR2        PIN_B2  
#define IR3        PIN_B4
#define IR4        PIN_B5

#use rs232(baud=9600, xmit=PIN_B1, rcv=PIN_B3)


// Macros to simplify I/O operations
//

#define LED_ON      output_high(LED)
#define LED_OFF     output_low(LED)

#define IR1_ON      output_high(IR1)
#define IR1_OFF     output_low(IR1)
#define IR2_ON      output_high(IR2)
#define IR2_OFF     output_low(IR2)
#define IR3_ON      output_high(IR3)
#define IR3_OFF     output_low(IR3)
#define IR4_ON      output_high(IR4)
#define IR4_OFF     output_low(IR4)

/******* Global Variables *******/
//Internal Variable to hold EEPROM data
//how many bytes are we actually using? (max 16)

//Data Format:

#define EEBYTES 8
unsigned int EEdata[EEBYTES];
// EEdata byte to store first 'local' IP
#define LocalLOC 3
// EEdata byte to store delay
#define DelayLOC 7

//DO NOT GIVE THESE INITIAL VALUES 
// unless you want those values selected
// ON EACH RESTART.
byte delay_remaining;            //Updating copy of EEdata[DelayLOC] for repeating delays
byte LEDselect;                  //Which IR LED should we drive next? [0-3]


/******** Helper Functions ******/
void irOff() {
  output_low(IR1);
  output_low(IR2);
  output_low(IR3);
  output_low(IR4);
}

void  irOn() {
 if (LEDselect == 0) output_high(IR1);
 if (LEDselect == 1) output_high(IR2);
 if (LEDselect == 2) output_high(IR3);
 if (LEDselect == 3) output_high(IR4);
}

/******** quarter-second "heartbeat" blink ********/

/*
 * Turn on the green LED for one quarter-second blink.
 * The duty cycle is specified as a value from 0-10:
 * 10 = 100% (all on), 1 = 10%, 0 = 0% (all off).
 */

void heartbeat(byte duty) 
{
  byte i;
  for (i = 0; i < 25; i++) 
  {
    restart_wdt();
    LED_ON;
    delay_ms(duty);    
    LED_OFF;
    delay_ms(10-duty);
  }
}

/******** Code to program internal EEPROM ***********************/

int _12c5_eeprom(int op, int EEADDR, int *p_dat);

// Functions to access internal EEPROM
//  (which appears to be an I2C device on PIN_B6 & PIN_B7)

#define WR_EE 1		
#define RD_EE 0

int _12c5_eeprom(int op, int EEADDR, int *p_dat)
{
   int COUNTER, PC_OFFSET, EEBYTE, EEDATA;
   if (op==WR_EE)
   {
      EEDATA=*p_dat;
      goto WRITE_BYTE;
   }
   goto READ_RANDOM;

#asm
READ_CURRENT:
    MOVLW   0x84    // PC offset for read current addr.  EE_OK bit7='1'
    MOVWF   PC_OFFSET    // Load PC offset
    GOTO    INIT_READ_CONTROL

WRITE_BYTE:
    MOVLW   0x80   // PC offset for write byte.  EE_OK: bit7 = '1'
    GOTO    INIT_WRITE_CONTROL

READ_RANDOM:
    MOVLW   0x83   // PC offset for read random.  EE_OK: bit7 = '1'

INIT_WRITE_CONTROL:
    MOVWF   PC_OFFSET     // Load PC offset register, value preset in W
    MOVLW   0xA0
    // Control byte with write bit, bit 0 = '0'

START_BIT:
    BCF     GPIO, SDA  // Start bit, SDA and SCL preset to '1'

//******* Set up output data (control, address, or data) and counter 
//*****
PREP_TRANSFER_BYTE:
    MOVWF   EEBYTE        // Byte to transfer to EEPROM already in W
    MOVLW   8            // Counter to transfer 8 bits
    MOVWF   COUNTER

//************  Clock out data (control, address, or data) byte  
//******
OUTPUT_BYTE:
        BCF     GPIO, SCL  // Set clock low during data set-up
        RLF     EEBYTE, F  // Rotate left, high order bit into carry bit
        BCF     GPIO, SDA  // Set data low, if rotated carry bit is
        BTFSC   STATUS, C  //   a '1', then:
        BSF     GPIO, SDA  // reset data pin to a one, otherwise leave low
        NOP
        BSF     GPIO, SCL  // clock data into EEPROM
        DECFSZ  COUNTER, F // Repeat until entire byte is sent
        GOTO    OUTPUT_BYTE
        NOP                // Needed to meet Timing (Thigh=4000nS)

//**********************  Acknowkedge Check 
//*******
        BCF     GPIO, SCL  // Set SCL low, 0.5us < ack valid < 3us
        NOP                // Needed to meet Timing (Tlow= 4700nS)
        BSF     GPIO, SDA
        NOP
        NOP
        NOP                // Necessary for SCL Tlow at low voltage,
        NOP                // Tlow=4700nS
        BSF     GPIO, SCL  // Raise SCL, EEPROM acknowledge still valid
        BTFSC   GPIO, SDA  // Check SDA for acknowledge (low)
        BCF     PC_OFFSET, 7 // If SDA not low (no ack), set error flag
        BCF     GPIO, SCL  // Lower SCL, EEPROM release bus
        BTFSS   PC_OFFSET, 7 // If no error continue, else stop bit
        GOTO    STOP_BIT

//*****  Set up program counter offset, based on EEPROM operating mode  
//*****
        MOVF    PC_OFFSET, W
        ANDLW   0x0F
        ADDWF   PCL, F
        GOTO    INIT_ADDRESS
        // PC offset=0, write control done, send address
        GOTO    INIT_WRITE_DATA
        // PC offset=1, write address done, send data
        GOTO    STOP_BIT
        // PC offset=2, write done, send stop bit
        GOTO    INIT_ADDRESS
        // PC offset=3, write control done, send address
        GOTO    INIT_READ_CONTROL
        // PC offset=4, send read control
        GOTO    READ_BIT_COUNTER
        // PC offset=5, set counter and read byte
        GOTO    STOP_BIT
        // PC offset=6, random read done, send stop
//**********  Initalize EEPROM data (address, data, or control) bytes  
//******
INIT_ADDRESS:
    INCF    PC_OFFSET, F // Increment PC offset to 2 (write) or to 4 (read)
    MOVF    EEADDR,W     // Put EEPROM address in W, ready to send to EEPROM
    GOTO    PREP_TRANSFER_BYTE

INIT_WRITE_DATA:
    INCF    PC_OFFSET, F // Increment PC offset to go to STOP_BIT next
    MOVF    EEDATA,W     // Put EEPROM data in W, ready to send to EEPROM
    GOTO    PREP_TRANSFER_BYTE

INIT_READ_CONTROL:
    BSF     GPIO, SCL // Raise SCL
    BSF     GPIO, SDA // raise SDA
    INCF    PC_OFFSET, F // Inc PC offset to go to READ_BIT_COUNTER next
    MOVLW   0xA1  // Set up read control byte, ready to send to EEPROM
    GOTO    START_BIT    //   bit 0 = '1' for read operation

//**************************  Read EEPROM data  
//*****
READ_BIT_COUNTER:
    BSF     GPIO, SDA // set data bit to 1 so we're not pulling bus down.
    NOP
    BSF     GPIO, SCL
    MOVLW   8           // Set counter so 8 bits will be read into EEDATA
    MOVWF   COUNTER

READ_BYTE:
    BSF     GPIO, SCL // Raise SCL, SDA valid.  SDA still input from ack
    BSF     STATUS, C              // Assume bit to be read = 1
    BTFSS   GPIO, SDA // Check if SDA = 1
    BCF     STATUS, C         // if SDA not = 1 then clear carry bit
    RLF     EEDATA, F // rotate carry bit (=SDA) into EEDATA;
    BCF     GPIO, SCL // Lower SCL
    BSF     GPIO, SDA // reset SDA
    DECFSZ  COUNTER, F //Decrement counter
    GOTO    READ_BYTE  // Read next bit if not finished reading byte

    BSF     GPIO, SCL
    NOP
    BCF     GPIO, SCL
//******************  Generate a STOP bit and RETURN  ***********************
//*****
STOP_BIT:
    BCF     GPIO, SDA // SDA=0, on TRIS, to prepare for transition to '1'
    BSF     GPIO, SCL // SCL = 1 to prepare for STOP bit
    NOP       // equivalent 4 NOPs neccessary for I2C spec Tsu:sto = 4.7us
    NOP
    NOP
    NOP
    BSF     GPIO, SDA // Stop bit, SDA transition to '1' while SCL high

#endasm
    *p_dat = EEDATA;
    return(PC_OFFSET);	// 1 is OK, 0 is NO
}


void LoadCodes() {
byte y,i;

 for (i=0;i< EEBYTES ;i++) {
  y=_12c5_eeprom(RD_EE, i, &EEdata[i]);
  delay_ms(10);		//You MUST delay between successive EEPROM Reads
 }

}

void SaveCodes() {
byte y,i;

 for (i=0;i< EEBYTES;i++) {
  y=_12c5_eeprom(WR_EE, i, &EEdata[i]);
  delay_ms(10);		//You MUST delay between successive EEPROM writes
 }
}

void PrintCodes() {
byte i;

 for (i=0;i< (EEBYTES-1);i++) {
  printf("%X,",EEdata[i]);
 }
  printf("%X.\r\n",EEdata[EEBYTES-1]);
}


/****************************/
void PowerOnBlink() {
  //This is only called after 2.3s WDT setting...

  delay_ms(200);

  Restart_WDT();

  LED_ON;
  delay_ms(150);
  LED_OFF;
}

void PowerOnMessage() {
  printf("\r\nSv2\r\n");
}


/*********** Delay commands **************/

void continue_delay() {
 byte x, time = 31; //See 12CE519.H for these codes

 for (x=7; x >= 0; x--) {
  if (bit_test(delay_remaining,x)) {
   if (!x) time--;
   bit_clear(delay_remaining,x);
   setup_counters(RTCC_INTERNAL, time/2);      //Configure WDT
   sleep(); 				    //Wait for restart
  }
  time = time-1;
 }//end for

 //delay_remaining was 0 and should have been trapped earlier.
 //This is bad. default to 2s
 setup_counters(RTCC_INTERNAL,WDT_2304MS);         
 sleep();
}


/*********** RS232 Configuration **************/

/* 
 getNum()
  Converts RS232 numeric input into byte.
  NO OVERFLOW CHECKING
*/
byte getNum(){
 int val=0;
 char in;

 do {
   in = getc();
	
   if (in >= '0' && in <= '9') {
    putc(in);
    val = val*10 + (in - 48);
   }

  } while (in != 13); //Enter

 return val;
}

/****/

void RS232_Config() {
 /*
  Turning off WDT fails, and actually forces a reset of the device.
  I'm assuming programming will be handled by an automated script,
  and 2s between entries will be more than enough time. 
  If this will be done by hand, well... figure out how to turn off 
  the WTD, or type quickly!
 */
 char in;

 Restart_WDT();

 in = getc();   
 //Wait for keyboard input
 //If no Keyboard Input, WDT will restart

 Restart_WDT();
 printf("\r");
 delay_ms(50);
 printf("P\r\n");


 //for each value we use in the EEPROM
 for (in=0; in < EEBYTES ;in++) {
  Restart_WDT();
  EEdata[in]  = getNum();
  //We really don't need these taking up space.
  // printf("\r\n");
 }

 Restart_WDT();
 SaveCodes();

 Restart_WDT();
 //To clear RS232 garbage from pulling pin low.
 printf("\r\n");
 PrintCodes();
 LED_off; //after printf

 //Wait for reboot
 sleep();
   
}

/*********** IR Transmition **************/
void IRburst(int D) {
 int i;

 for (i=0; i<10; i++) { 
  irOn();
  DELAY_US(6); 
  irOff();
  DELAY_US(11); 
 }		
                  			/* Bit is pause between flashes */
					/* This conserves power */
 if (D == 0) {				/* IRstartbit = 1650us */
  DELAY_MS(1);
  DELAY_US(250);
  DELAY_US(250);
  DELAY_US(150);
 } else if (D==1) {		/* IRonebit = 1100us */
  DELAY_MS(1);
  DELAY_US(100);
 } else if (D==2) {		/* IRzerobit = 550us */
  DELAY_US(250);		/* PCM problem with # > 255 */
  DELAY_US(250);
  DELAY_US(50);
 } else if (D==3) {		/* IRstopbit = 100us */
  DELAY_US(100);
 }

}

#define IRstartbit IRburst(0)
#define IRonebit   IRburst(1)
#define IRzerobit  IRburst(2)
#define IRstopbit  IRburst(3)

// rewritten by JAW 
void IRsendbyte(int J) {
int i;
 IRstartbit;
 for (i=0;i<8;i++) {
  if (J&0x01) IRonebit; else IRzerobit;
  J >>= 1;
 }
 IRstopbit;
} 

/* Send the IR information packet */
send_packet() {				
 
int x;
int ircheck = 1;
 
 Restart_WDT();
 IRsendbyte(1);			/* First send a one */

 for(x=0; x<3; x++){		/* Send the first three bytes of info */
  Restart_WDT();
  IRsendbyte(EEdata[x]);
  ircheck = ircheck^EEdata[x];
 } 
 Restart_WDT();			          //This is based on which LED is firing
 IRsendbyte(EEdata[LEDselect + LocalLOC ]);
 ircheck = ircheck^EEdata[LEDselect + LocalLOC ];

 Restart_WDT();
 if (ircheck == 1) ircheck = 2;
 IRsendbyte(ircheck);		/* Last, the error checking byte */

 //Configure next LED to flash
 LEDselect = (LEDselect+1) % 4 ;
 if (!LEDselect) 
   {
     heartbeat(2);
   }
}


/*********** MAIN ****************/
void main() { 
 int cause;
  
  //We need to snag this before configuration below clobbers it.
  cause = restart_cause();
  LED_OFF;                                        //Because of useRS232 above, pin is ON by default.

  #ifndef OTP
   //this should NOT be used for OTP chips
   SET_OSCILLATOR;				  //Fix Oscillator offset, now clock timing is correct
  #endif

  set_tris_b(IRX_B_TRIS);			  //Set IO Pins Correctly.
  setup_counters(RTCC_INTERNAL, WDT_2304MS);      //Must Set internal timer to use Pin_B2
  restart_WDT();

  if (cause == NORMAL_POWER_UP) {
    PowerOnBlink();
    PowerOnMessage();

    //Setup Initial variables
    // Variables are initialized here, not where they are defined, since
    // we want these values for the FIRST reset, not EVERY reset.
    delay_remaining=0;
    LEDselect=0;

    //This only needs to be run once, data survives sleep/WDT reset
    Restart_WDT();
    LoadCodes();

    Restart_WDT();
    PrintCodes();
    LED_off;           //Turn LED off during keyboard delay
    
    //Gives user 2.3s to start EEPROM write.
    RS232_Config();
  } else {
    //PIC restarted from WDT
    if (delay_remaining == 0) {
      //Time to flash
      delay_remaining = EEdata[ DelayLOC ];
      send_packet();
      continue_delay();
    } else {
      //Go back to sleep
      continue_delay();
    }
  } 
}



